(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs'), require('@angular/common'), require('@angular/forms'), require('@angular/platform-browser')) :
    typeof define === 'function' && define.amd ? define('angular-tree-grid', ['exports', '@angular/core', 'rxjs', '@angular/common', '@angular/forms', '@angular/platform-browser'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['angular-tree-grid'] = {}, global.ng.core, global.rxjs, global.ng.common, global.ng.forms, global.ng.platformBrowser));
}(this, (function (exports, i0, rxjs, common, forms, platformBrowser) { 'use strict';

    var AngularTreeGridService = /** @class */ (function () {
        function AngularTreeGridService() {
            this.display_data_observable = new rxjs.Subject();
            this.display_data_observable$ = this.display_data_observable.asObservable();
        }
        AngularTreeGridService.prototype.updateDisplayDataObservable = function (display_data) {
            this.display_data_observable.next(display_data);
        };
        AngularTreeGridService.prototype.findRowIndex = function (display_data, configs, row_id) {
            return display_data.map(function (row) { return row[configs.id_field]; }).
                indexOf(row_id);
        };
        AngularTreeGridService.prototype.selectAll = function (display_data) {
            display_data.forEach(function (data) {
                data.row_selected = true;
            });
        };
        AngularTreeGridService.prototype.deSelectAll = function (display_data) {
            display_data.forEach(function (data) {
                data.row_selected = false;
            });
        };
        AngularTreeGridService.prototype.expandAll = function (expand_tracker) {
            for (var key in expand_tracker) {
                if (expand_tracker.hasOwnProperty(key)) {
                    expand_tracker[key] = true;
                }
            }
        };
        AngularTreeGridService.prototype.collapseAll = function (expand_tracker) {
            for (var key in expand_tracker) {
                if (expand_tracker.hasOwnProperty(key)) {
                    expand_tracker[key] = false;
                }
            }
            expand_tracker[''] = true;
        };
        AngularTreeGridService.prototype.expandRow = function (row_id, expand_tracker, expand_event, suppress_event, configs, display_data, store) {
            if (configs.subgrid) {
                this.expandSubgridRow(row_id, expand_tracker, expand_event, suppress_event, configs, display_data, store);
                return;
            }
            var row_index = this.findRowIndex(display_data, configs, row_id);
            var row_data = display_data[row_index];
            var pathx = row_data.pathx;
            var parts = pathx.split('.');
            expand_tracker[row_data.pathx] = true;
            var expanded_count = 1;
            for (var index = 0; index < display_data.length; index++) {
                var record = display_data[index];
                // Stop when all rows are expanded.
                if (expanded_count >= parts.length) {
                    return;
                }
                // Join paths as we expand.
                var key = parts.slice(0, expanded_count).join('.');
                // We don't want to expand it's children here.
                if (record.pathx.indexOf(key) !== -1) {
                    expanded_count += 1;
                    expand_tracker[record.pathx] = true;
                    if (!suppress_event) {
                        if (configs.load_children_on_expand) {
                            this.emitExpandRowEvent(expand_tracker, expand_event, store, row_data, configs);
                        }
                        else {
                            expand_event.emit({ event: null, data: row_data });
                        }
                    }
                }
            }
        };
        AngularTreeGridService.prototype.collapseRow = function (row_id, expand_tracker, collapse_event, suppress_event, configs, display_data) {
            var row_index = this.findRowIndex(display_data, configs, row_id);
            var row_data = display_data[row_index];
            var pathx = row_data.pathx;
            expand_tracker[pathx] = false;
            // Collapse children rows as well
            display_data.forEach(function (record) {
                if (record.pathx.indexOf(pathx) !== -1) {
                    expand_tracker[record.pathx] = 0;
                    if (!suppress_event) {
                        collapse_event.emit({ event: null, data: row_data });
                    }
                }
            });
        };
        AngularTreeGridService.prototype.emitExpandRowEvent = function (expand_tracker, expand, store, row_data, configs) {
            var promise = new Promise(function (resolve, reject) {
                expand.emit({
                    data: row_data,
                    resolve: resolve
                });
            });
            expand_tracker[row_data.pathx] = true;
            store.remove_children(row_data);
            row_data.is_loading = true;
            // Add Child rows to the table.
            promise.then(function (child_rows) {
                row_data.is_loading = false;
                store.remove_children(row_data);
                if (child_rows) {
                    child_rows.map(function (child) {
                        child.leaf = true;
                        child.levelx = row_data.levelx + 1;
                        child.pathx = row_data.pathx + '.' + child[configs.id_field];
                        child.parent_pathx = row_data.pathx;
                        child[configs.parent_id_field] = row_data[configs.id_field];
                    });
                    store.add_children(row_data, child_rows);
                }
            }).catch(function (err) { });
        };
        AngularTreeGridService.prototype.disableRowSelection = function (display_data, configs, row_id) {
            var row_index = this.findRowIndex(display_data, configs, row_id);
            display_data[row_index].selection_disabled = true;
        };
        AngularTreeGridService.prototype.enableRowSelection = function (display_data, configs, row_id) {
            var row_index = this.findRowIndex(display_data, configs, row_id);
            display_data[row_index].selection_disabled = false;
        };
        AngularTreeGridService.prototype.disableRowExpand = function (display_data, configs, row_id) {
            var row_index = this.findRowIndex(display_data, configs, row_id);
            display_data[row_index].expand_disabled = true;
        };
        AngularTreeGridService.prototype.enableRowExpand = function (display_data, configs, row_id) {
            var row_index = this.findRowIndex(display_data, configs, row_id);
            display_data[row_index].expand_disabled = false;
        };
        AngularTreeGridService.prototype.expandSubgridRow = function (row_id, expand_tracker, expand_event, suppress_event, configs, display_data, store) {
            var row_index = this.findRowIndex(display_data, configs, row_id);
            var row_data = display_data[row_index];
            expand_tracker[row_data.pathx] = true;
            if (!suppress_event) {
                this.emitSubgridExpandRowEvent(expand_tracker, expand_event, store, row_data);
            }
        };
        AngularTreeGridService.prototype.emitSubgridExpandRowEvent = function (expand_tracker, expand, store, row_data) {
            var promise = new Promise(function (resolve, reject) {
                expand.emit({
                    data: row_data,
                    resolve: resolve
                });
            });
            expand_tracker[row_data.pathx] = true;
            var blank_row = store.showBlankRow(row_data);
            blank_row.loading_children = true;
            // Add Child rows to the table.
            promise.then(function (child_rows) {
                blank_row.loading_children = false;
                if (child_rows) {
                    child_rows.map(function (child) {
                        child.leaf = true;
                    });
                    blank_row.children = child_rows;
                }
                else {
                    // Persist old children. If didn't exist then assign blank array.
                    if (!blank_row.children) {
                        blank_row.children = [];
                    }
                }
            }).catch(function (err) { });
        };
        return AngularTreeGridService;
    }());
    AngularTreeGridService.ɵprov = i0.ɵɵdefineInjectable({ factory: function AngularTreeGridService_Factory() { return new AngularTreeGridService(); }, token: AngularTreeGridService, providedIn: "root" });
    AngularTreeGridService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    AngularTreeGridService.ctorParameters = function () { return []; };

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        var desc = Object.getOwnPropertyDescriptor(m, k);
        if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
            desc = { enumerable: true, get: function () { return m[k]; } };
        }
        Object.defineProperty(o, k2, desc);
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2)
            for (var i = 0, l = from.length, ar; i < l; i++) {
                if (ar || !(i in from)) {
                    if (!ar)
                        ar = Array.prototype.slice.call(from, 0, i);
                    ar[i] = from[i];
                }
            }
        return to.concat(ar || Array.prototype.slice.call(from));
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, state, kind, f) {
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    }
    function __classPrivateFieldSet(receiver, state, value, kind, f) {
        if (kind === "m")
            throw new TypeError("Private method is not writable");
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    }
    function __classPrivateFieldIn(state, receiver) {
        if (receiver === null || (typeof receiver !== "object" && typeof receiver !== "function"))
            throw new TypeError("Cannot use 'in' operator on non-object");
        return typeof state === "function" ? receiver === state : state.has(receiver);
    }

    var Store = /** @class */ (function () {
        function Store(angularTreeGridService) {
            this.angularTreeGridService = angularTreeGridService;
        }
        Store.prototype.getRawData = function () {
            return this.raw_data;
        };
        Store.prototype.setRawData = function (raw_data) {
            this.raw_data = raw_data;
        };
        Store.prototype.getProcessedData = function () {
            return this.processed_data;
        };
        Store.prototype.setProcessedData = function (processed_data) {
            this.processed_data = processed_data;
            this.setDisplayData(__spread(processed_data));
        };
        Store.prototype.getDisplayData = function () {
            return this.display_data;
        };
        Store.prototype.setDisplayData = function (display_data) {
            this.display_data = display_data;
            this.angularTreeGridService.updateDisplayDataObservable(this.display_data);
        };
        /**
         * Show Blank row for subgrid.
         *
         * @param  row_data Row Data
         * @returns blank_row
         */
        Store.prototype.showBlankRow = function (row_data) {
            var _this = this;
            var row_index = this.display_data.map(function (row) { return row[_this.configs.id_field]; }).
                indexOf(row_data[this.configs.id_field]);
            var blank_row = this.display_data[row_index + 1];
            if (!blank_row || blank_row.parent_pathx !== row_data[this.configs.id_field]) {
                blank_row = {
                    leaf: true,
                    row_selected: true,
                    parent_pathx: row_data[this.configs.id_field]
                };
                blank_row[this.configs.id_field] = -1;
                this.display_data.splice(row_index + 1, 0, blank_row);
            }
            return blank_row;
        };
        Store.prototype.remove_children = function (row_data) {
            var new_processed_data = [];
            for (var index = 0; index < this.processed_data.length; index++) {
                var element = this.processed_data[index];
                if (element[this.configs.parent_id_field] !== row_data[this.configs.id_field]) {
                    new_processed_data.push(element);
                }
            }
            this.setProcessedData(new_processed_data);
        };
        Store.prototype.add_children = function (row_data, children) {
            var _this = this;
            var row_index = this.processed_data.map(function (row) { return row[_this.configs.id_field]; }).
                indexOf(row_data[this.configs.id_field]);
            var top_rows = this.processed_data.slice(0, row_index + 1);
            var bottom_rows = this.processed_data.slice(row_index + 1);
            this.processed_data = top_rows.concat(children).concat(bottom_rows);
            this.setDisplayData(__spread(this.processed_data));
            this.angularTreeGridService.updateDisplayDataObservable(this.display_data);
        };
        Store.prototype.filterBy = function (columns, search_values) {
            this.display_data = this.processed_data.filter(function (record) {
                var found = true;
                for (var index = 0; index < columns.length; index++) {
                    var column = columns[index];
                    var column_value = record[column.name];
                    var search_value = search_values[index];
                    // If blank then continue.
                    if (!search_value) {
                        continue;
                    }
                    // Call custom filter function.
                    if (column.filter_function) {
                        var response = column.filter_function(record, column, column_value, search_value);
                        if (response === false) {
                            found = false;
                        }
                    }
                    else {
                        if (typeof (column_value) === 'number') {
                            if (column_value !== parseInt(search_value, 10)) {
                                found = false;
                            }
                        }
                        else {
                            if (!column.case_sensitive_filter) {
                                column_value = column_value.toLowerCase();
                                search_value = search_value.toLowerCase();
                            }
                            if (column_value.indexOf(search_value) === -1) {
                                found = false;
                            }
                        }
                    }
                }
                return found;
            });
            this.angularTreeGridService.updateDisplayDataObservable(this.display_data);
        };
        Store.prototype.findTopParentNode = function (data, configs) {
            var ids = data.map(function (element) { return element[configs.id_field]; });
            var top_parents = [];
            // Find parents ie if parent_id is not present in ids.
            data.forEach(function (element) {
                if (ids.indexOf(element[configs.parent_id_field]) === -1) {
                    top_parents.push(element[configs.parent_id_field]);
                }
            });
            // Remove duplicates
            top_parents = top_parents.filter(function (item, pos, self) {
                return self.indexOf(item) === pos;
            });
            return top_parents;
        };
        Store.prototype.processData = function (data, expand_tracker, configs, edit_tracker, internal_configs) {
            var _this = this;
            var top_parents = this.findTopParentNode(data, configs);
            var processed_data = [];
            internal_configs.top_parents = top_parents;
            data.map(function (rec) {
                rec.pathx = [];
                rec.leaf = false;
            });
            top_parents.forEach(function (top_parent) {
                var children = _this.findChildren(data, top_parent, configs);
                _this.orderData(data, processed_data, configs, children, [], 0);
            });
            processed_data.map(function (rec) {
                var parent_pathx = rec.parent_pathx;
                rec.parent_pathx = parent_pathx.join('.');
                parent_pathx.push(rec[configs.id_field]);
                // Add current id to create current path.
                rec.pathx = parent_pathx.join('.');
                expand_tracker[rec.pathx] = false;
                edit_tracker[rec[configs.id_field]] = false;
                // For Subgrid feature, expect all rows are expandable.
                if (configs.subgrid) {
                    rec.leaf = false;
                }
            });
            // Expand root.
            expand_tracker[''] = true;
            this.setProcessedData(processed_data);
            this.setRawData(data);
            this.configs = configs;
        };
        Store.prototype.findChildren = function (data, id, configs) {
            return data.filter(function (rec) { return rec[configs.parent_id_field] === id; });
        };
        Store.prototype.orderData = function (data, processed_data, configs, parents, paths, levelx) {
            var _this = this;
            parents.forEach(function (parent) {
                var children = _this.findChildren(data, parent[configs.id_field], configs);
                if (children.length === 0) {
                    parent.leaf = true;
                    parent.levelx = levelx;
                    parent.parent_pathx = __spread(paths);
                    processed_data.push(parent);
                }
                else {
                    parent.parent_pathx = __spread(paths);
                    parent.levelx = levelx;
                    processed_data.push(parent);
                    var newPaths = __spread(paths, [parent[configs.id_field]]);
                    _this.orderData(data, processed_data, configs, children, newPaths, levelx + 1);
                }
            });
        };
        Store.prototype.refreshDisplayData = function () {
            this.display_data = this.processed_data;
            this.angularTreeGridService.updateDisplayDataObservable(this.display_data);
        };
        return Store;
    }());

    var AngularTreeGridComponent = /** @class */ (function () {
        function AngularTreeGridComponent(angularTreeGridService) {
            this.angularTreeGridService = angularTreeGridService;
            this.processed_data = [];
            this.expand_tracker = {};
            this.columns = [];
            this.edit_tracker = {}; // Track Edit options.
            this.internal_configs = {
                show_add_row: false,
                show_parent_col: false,
                all_selected: false
            };
            this.store = new Store(this.angularTreeGridService);
            this.default_configs = {
                css: {
                    expand_icon: '',
                    collapse_icon: '',
                    add_icon: '',
                    edit_icon: '',
                    delete_icon: '',
                    save_icon: '',
                    cancel_icon: '',
                    row_selection_class: 'selected',
                    header_class: '',
                    row_filter_class: '',
                    table_class: ''
                },
                actions: {
                    edit: false,
                    add: false,
                    delete: false,
                    resolve_edit: false,
                    resolve_add: false,
                    resolve_delete: false,
                    edit_parent: false
                },
                data_loading_text: 'Loading...',
                filter: false,
                multi_select: false,
                show_parent_on_edit: true,
                show_summary_row: false,
                subgrid: false,
                load_children_on_expand: false,
                rtl_direction: false,
                action_column_width: '60px',
                row_class_function: function () { return ''; },
                row_edit_function: function () { return true; },
                row_delete_function: function () { return true; },
                subgrid_config: {
                    show_summary_row: false,
                    data_loading_text: 'Loading...'
                }
            };
            this.default_column_config = {
                sorted: 0,
                sort_type: null,
                editable: false,
                hidden: false,
                filter: true,
                case_sensitive_filter: false
            };
            this.cellclick = new i0.EventEmitter();
            this.expand = new i0.EventEmitter();
            this.collapse = new i0.EventEmitter();
            this.rowselect = new i0.EventEmitter();
            this.rowdeselect = new i0.EventEmitter();
            this.rowselectall = new i0.EventEmitter();
            this.rowdeselectall = new i0.EventEmitter();
            this.rowadd = new i0.EventEmitter();
            this.rowsave = new i0.EventEmitter();
            this.rowdelete = new i0.EventEmitter();
        }
        AngularTreeGridComponent.prototype.ngOnInit = function () {
            if (!this.validateConfigs()) {
                return;
            }
            this.setDefaultConfigs();
            this.setColumnNames();
        };
        AngularTreeGridComponent.prototype.ngOnChanges = function () {
            if (!this.validateConfigs()) {
                return;
            }
            this.setDefaultConfigs();
            this.setColumnNames();
            this.store.processData(this.data, this.expand_tracker, this.configs, this.edit_tracker, this.internal_configs);
        };
        AngularTreeGridComponent.prototype.validateConfigs = function () {
            if (!this.data) {
                window.console.warn('data can\'t be empty!');
                return;
            }
            if (!this.configs) {
                window.console.warn('configs can\'t be empty!');
                return;
            }
            var element = this.data[0];
            var isValidated = true;
            if (!this.configs.id_field) {
                isValidated = false;
                window.console.error('id field is mandatory!');
            }
            if (!this.configs.parent_id_field && !this.configs.subgrid) {
                isValidated = false;
                window.console.error('parent_id field is mandatory!');
            }
            if (element && !element.hasOwnProperty(this.configs.id_field)) {
                isValidated = false;
                console.error('id_field doesn\'t exist!');
            }
            if (element && !element.hasOwnProperty(this.configs.parent_id_field)
                && !this.configs.subgrid
                && !this.configs.load_children_on_expand) {
                isValidated = false;
                console.error('parent_id_field doesn\'t exist!');
            }
            if (element && !element.hasOwnProperty(this.configs.parent_display_field)) {
                isValidated = false;
                console.error('parent_display_field doesn\'t exist! Basically this field will be expanded.');
            }
            if (this.configs.subgrid && !this.configs.subgrid_config) {
                isValidated = false;
                console.error('subgrid_config doesn\'t exist!');
            }
            if (this.configs.subgrid && this.configs.subgrid_config && !this.configs.subgrid_config.id_field) {
                isValidated = false;
                console.error('id_field of subgrid doesn\'t exist!');
            }
            if (this.configs.subgrid && this.configs.subgrid_config && !this.configs.subgrid_config.columns) {
                isValidated = false;
                console.error('columns of subgrid doesn\'t exist!');
            }
            return isValidated;
        };
        AngularTreeGridComponent.prototype.setDefaultConfigs = function () {
            this.processed_data = [];
            this.configs = Object.assign({}, this.default_configs, this.configs);
            // Deep clone.
            this.configs.actions = Object.assign({}, this.default_configs.actions, this.configs.actions);
            this.configs.css = Object.assign({}, this.default_configs.css, this.configs.css);
            this.configs.subgrid_config = Object.assign({}, this.default_configs.subgrid_config, this.configs.subgrid_config);
            if (this.configs.subgrid) {
                this.configs.actions.add = false;
            }
        };
        AngularTreeGridComponent.prototype.setColumnNames = function () {
            var _this = this;
            this.columns = this.configs.columns ? this.configs.columns : [];
            // If columns doesn't exist in user's object.
            if (!this.configs.columns) {
                var column_keys = Object.keys(this.data[0]);
                // Insert Header and default configuration.
                column_keys.forEach(function (key) {
                    _this.columns.push(Object.assign({ 'header': key, 'name': key }, _this.default_column_config));
                });
            }
            else {
                // Insert Header and default configuration.
                for (var i = 0; i < this.columns.length; i++) {
                    this.columns[i] = Object.assign({}, this.default_column_config, this.columns[i]);
                }
            }
        };
        AngularTreeGridComponent.prototype.collapseAll = function () {
            this.angularTreeGridService.collapseAll(this.expand_tracker);
        };
        AngularTreeGridComponent.prototype.expandAll = function () {
            this.angularTreeGridService.expandAll(this.expand_tracker);
        };
        AngularTreeGridComponent.prototype.selectAll = function () {
            this.angularTreeGridService.selectAll(this.store.getDisplayData());
            this.internal_configs.all_selected = true;
        };
        AngularTreeGridComponent.prototype.deSelectAll = function () {
            this.angularTreeGridService.deSelectAll(this.store.getDisplayData());
            this.internal_configs.all_selected = false;
        };
        AngularTreeGridComponent.prototype.expandRow = function (row_id, suppress_event) {
            this.angularTreeGridService.expandRow(row_id, this.expand_tracker, this.expand, suppress_event, this.configs, this.store.getDisplayData(), this.store);
        };
        AngularTreeGridComponent.prototype.collapseRow = function (row_id, suppress_event) {
            this.angularTreeGridService.collapseRow(row_id, this.expand_tracker, this.collapse, suppress_event, this.configs, this.store.getDisplayData());
        };
        AngularTreeGridComponent.prototype.disableRowSelection = function (row_id) {
            this.angularTreeGridService.disableRowSelection(this.store.getDisplayData(), this.configs, row_id);
        };
        AngularTreeGridComponent.prototype.enableRowSelection = function (row_id) {
            this.angularTreeGridService.enableRowSelection(this.store.getDisplayData(), this.configs, row_id);
        };
        AngularTreeGridComponent.prototype.disableRowExpand = function (row_id) {
            this.angularTreeGridService.disableRowExpand(this.store.getDisplayData(), this.configs, row_id);
        };
        AngularTreeGridComponent.prototype.enableRowExpand = function (row_id) {
            this.angularTreeGridService.enableRowExpand(this.store.getDisplayData(), this.configs, row_id);
        };
        return AngularTreeGridComponent;
    }());
    AngularTreeGridComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'db-angular-tree-grid',
                    template: "<table [attr.class]=\"'db-tree-view ' + configs.css.table_class\" >\n    <thead \n        db-tree-head\n        [store]=\"store\"\n        [expand_tracker]=\"expand_tracker\"\n        [internal_configs]=\"internal_configs\"\n        [edit_tracker]=\"edit_tracker\"\n        [rowselectall]=\"rowselectall\"\n        [rowdeselectall]=\"rowdeselectall\"\n        [columns]=\"columns\"\n        [configs]=\"configs\">\n    </thead>\n\n    <tbody \n        db-tree-body\n        [store]=\"store\"\n        [expand_tracker]=\"expand_tracker\"\n        [edit_tracker]=\"edit_tracker\"\n        [internal_configs]=\"internal_configs\"\n        [columns]=\"columns\"\n        [configs]=\"configs\"\n        [cellclick]=\"cellclick\"\n        [expand]=\"expand\"\n        [collapse]=\"collapse\"\n        [rowdelete]=\"rowdelete\"\n        [rowsave]=\"rowsave\"\n        [rowadd]=\"rowadd\"\n        [rowselect]=\"rowselect\"\n        [rowdeselect]=\"rowdeselect\"\n    >        \n    </tbody>\n</table>",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}"]
                },] }
    ];
    AngularTreeGridComponent.ctorParameters = function () { return [
        { type: AngularTreeGridService }
    ]; };
    AngularTreeGridComponent.propDecorators = {
        data: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        cellclick: [{ type: i0.Output }],
        expand: [{ type: i0.Output }],
        collapse: [{ type: i0.Output }],
        rowselect: [{ type: i0.Output }],
        rowdeselect: [{ type: i0.Output }],
        rowselectall: [{ type: i0.Output }],
        rowdeselectall: [{ type: i0.Output }],
        rowadd: [{ type: i0.Output }],
        rowsave: [{ type: i0.Output }],
        rowdelete: [{ type: i0.Output }]
    };

    var TreeBodyComponent = /** @class */ (function () {
        function TreeBodyComponent(angularTreeGridService) {
            this.angularTreeGridService = angularTreeGridService;
            this.parents = [];
        }
        TreeBodyComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.display_data = this.store.getDisplayData();
            this.angularTreeGridService.display_data_observable$.subscribe(function (store) {
                _this.display_data = _this.store.getDisplayData();
                _this.setParents();
            });
            this.setParents();
        };
        TreeBodyComponent.prototype.setParents = function () {
            var _this = this;
            this.parents = this.store.raw_data.map(function (element) {
                return {
                    'id': element[_this.configs.id_field],
                    'value': element[_this.configs.parent_display_field]
                };
            });
        };
        TreeBodyComponent.prototype.refreshData = function (element) {
            // If edit parent is not true then don't refresh.
            if (!this.configs.actions.edit_parent) {
                return;
            }
            element[this.configs.parent_id_field] = parseInt(element[this.configs.parent_id_field], 10);
            this.expand_tracker = {};
            this.edit_tracker = {};
            this.store.processData(this.store.getRawData(), this.expand_tracker, this.configs, this.edit_tracker, this.internal_configs);
        };
        TreeBodyComponent.prototype.onRowExpand = function (event) {
            var row_data = event.data;
            if (!this.configs.load_children_on_expand) {
                this.expand_tracker[row_data.pathx] = true;
                this.expand.emit(event);
            }
            else {
                this.angularTreeGridService.emitExpandRowEvent(this.expand_tracker, this.expand, this.store, row_data, this.configs);
            }
        };
        TreeBodyComponent.prototype.onRowCollapse = function (event) {
            var _this = this;
            var row_data = event.data;
            this.expand_tracker[row_data.pathx] = false;
            // Collapse all of its children.
            var keys = Object.keys(this.expand_tracker);
            keys.forEach(function (key) {
                if (key.indexOf(row_data.pathx) !== -1) {
                    _this.expand_tracker[key] = 0;
                }
            });
            this.collapse.emit(event);
        };
        TreeBodyComponent.prototype.saveRecord = function ($event) {
            var _this = this;
            var element = $event.data;
            if (this.configs.actions.resolve_edit) {
                var promise = new Promise(function (resolve, reject) {
                    _this.rowsave.emit({
                        data: element,
                        resolve: resolve
                    });
                });
                promise.then(function () {
                    _this.checkAndRefreshData(element);
                }).catch(function (err) { });
            }
            else {
                this.checkAndRefreshData(element);
                this.rowsave.emit(element);
            }
        };
        TreeBodyComponent.prototype.checkAndRefreshData = function (element) {
            this.edit_tracker[element[this.configs.id_field]] = false;
            this.internal_configs.show_parent_col = false;
            // Only refresh if Parent has been changed.
            if (this.internal_configs.current_edited_row[this.configs.parent_id_field]
                !== element[this.configs.parent_id_field]) {
                this.refreshData(element);
            }
        };
        TreeBodyComponent.prototype.addRow = function (element) {
            var _this = this;
            if (this.configs.actions.resolve_add) {
                var promise = new Promise(function (resolve, reject) {
                    _this.rowadd.emit({
                        data: element,
                        resolve: resolve
                    });
                });
                promise.then(function () {
                    _this.internal_configs.show_add_row = false;
                    _this.refreshData(element);
                }).catch(function (err) { });
            }
            else {
                this.refreshData(element);
                this.internal_configs.show_add_row = false;
                this.rowadd.emit(element);
            }
        };
        TreeBodyComponent.prototype.cancelEdit = function (row_data) {
            var index = row_data[this.configs.id_field];
            // Cancel all changes ie copy from back up.
            Object.assign(row_data, this.internal_configs.current_edited_row);
            this.edit_tracker[index] = false;
            this.internal_configs.show_parent_col = false;
        };
        TreeBodyComponent.prototype.selectRow = function (row_data, event) {
            // Don't run if Multi select is enabled.
            if (this.configs.multi_select) {
                return;
            }
            this.store.getDisplayData().forEach(function (data) {
                data.row_selected = false;
            });
            row_data.row_selected = true;
            this.rowselect.emit({ data: row_data, event: event });
        };
        TreeBodyComponent.prototype.selectRowOnCheck = function (row_data, event) {
            if (event.target.checked) {
                row_data.row_selected = true;
                this.rowselect.emit({ data: row_data, event: event });
            }
            else {
                row_data.row_selected = false;
                this.rowdeselect.emit({ data: row_data, event: event });
            }
            this.setSelectAllConfig();
        };
        /**
         * Set Select All config on Select change.
         *
         */
        TreeBodyComponent.prototype.setSelectAllConfig = function () {
            var select_all = true;
            this.store.getDisplayData().forEach(function (data) {
                if (!data.row_selected) {
                    select_all = false;
                }
            });
            this.internal_configs.all_selected = select_all;
        };
        return TreeBodyComponent;
    }());
    TreeBodyComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-tree-body]',
                    template: "<ng-container *ngIf=\"configs\">\n<tr *ngIf=\"store.raw_data.length==0\">\n  <td [innerHTML]=\"configs.data_loading_text\" [attr.colspan]=\"columns.length + 1\" style=\"text-align: center\"></td>\n</tr>\n<tr db-filter-row \n  [columns]=\"columns\" \n  [configs]=\"configs\"\n  [store]=\"store\"\n  [internal_configs]=\"internal_configs\" \n  *ngIf=\"configs.filter\"\n  [expand_tracker]=\"expand_tracker\"\n  [ngClass]=\"configs.css.row_filter_class\">\n</tr>\n<tr db-add-row \n  [columns]=\"columns\" \n  [configs]=\"configs\" \n  [internal_configs]=\"internal_configs\" \n  [store]=\"store\"\n  (rowadd)=\"addRow($event)\"\n  *ngIf=\"internal_configs.show_add_row\"\n  [ngClass]=\"configs.row_class_function() + ' row-add'\">\n</tr>\n<ng-container *ngIf=\"configs.subgrid\">\n  <tr db-subgrid\n  *ngFor=\"let row_data of display_data\"\n  class=\"subgrid-row\"\n  [configs]=\"configs\" \n  [internal_configs]=\"internal_configs\" \n  [expand_tracker]=\"expand_tracker\" \n  [edit_tracker]=\"edit_tracker\" \n  [store]=\"store\"\n  [row_data]=\"row_data\"\n  [cellclick]=\"cellclick\"\n  [rowselect]=\"rowselect\"\n  [rowdeselect]=\"rowdeselect\"\n  [expand]=\"expand\"\n  [rowsave]=\"rowsave\"\n  [rowdelete]=\"rowdelete\"\n  >\n\n  </tr>\n</ng-container>\n<ng-container *ngIf=\"!configs.subgrid\">\n  <tr \n  *ngFor=\"let row_data of display_data\"\n  [attr.class]=\"configs.row_class_function(row_data) + ' ' + (row_data.row_selected ? configs.css.row_selection_class : '')\"\n  (click)=\"selectRow(row_data, $event)\" \n  >  \n  <ng-container *ngIf=\"expand_tracker[row_data.parent_pathx]\">\n    <td *ngIf=\"configs.multi_select\" class=\"checkbox_column\">\n      <input type=\"checkbox\" [checked]=\"row_data.row_selected\" (click)=\"selectRowOnCheck(row_data, $event)\" \n        [attr.disabled]=\"row_data.selection_disabled\">\n    </td>\n    <td db-tree-cell-actions \n      *ngIf=\"(configs.actions.edit || configs.actions.delete || configs.actions.add)\"\n      [row_data]=\"row_data\"\n      [configs]=\"configs\"\n      [store]=\"store\"\n      [edit_tracker]=\"edit_tracker\"\n      [internal_configs]=\"internal_configs\"\n      [rowdelete]=\"rowdelete\"\n      (canceledit)=\"cancelEdit($event)\" \n      (editcomplete)=\"saveRecord($event)\">\n    </td>\n    <td *ngFor=\"let column of columns; index as i\" \n    class=\"{{column.css_class}}\"\n    [ngClass]=\"{'column-hide': column.hidden, 'expand-column': i == 0}\">\n      <db-tree-cell\n        [configs]=\"configs\"\n        [column]=\"column\"\n        [index]=\"i\"\n        [row_data]=\"row_data\"\n        [expand_tracker]=\"expand_tracker\"\n        [edit_on]=\"edit_tracker[row_data[configs.id_field]]\"\n        [cellclick]=\"cellclick\"\n        (rowexpand)=\"onRowExpand($event)\"\n        (rowcollapse)=\"onRowCollapse($event)\"\n        (editcomplete)=\"saveRecord($event)\"\n      >\n      </db-tree-cell>\n    </td>\n    <td *ngIf=\"configs.show_parent_on_edit && internal_configs.show_parent_col\">\n      <select *ngIf=\"edit_tracker[row_data[configs.id_field]]\" \n        [(ngModel)]=\"row_data[configs.parent_id_field]\">\n        <option *ngFor=\"let parent of parents\" [value]=\"parent.id\">{{parent.value}}</option>\n      </select>\n    </td>    \n\n    <!-- Add column to fix UI issue when add row is enabled but don't show when edit is enabled for the row -->\n    <td *ngIf=\"internal_configs.show_add_row && !(internal_configs.show_parent_col && configs.show_parent_on_edit)\"></td>\n  </ng-container>\n</tr>\n<tr *ngIf=\"configs.show_summary_row\">\n  <td *ngIf=\"configs.multi_select\"></td>\n  <td *ngIf=\"(configs.actions.edit || configs.actions.delete || configs.actions.add)\"></td>\n  <td *ngFor=\"let column of configs.columns\">\n    <div [innerHTML]=\"column.summary_renderer && column.summary_renderer(display_data)\"></div>\n  </td>\n</tr>\n</ng-container>\n\n</ng-container>\n",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{border-bottom:1px solid #cdd5dc}tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr.subgrid-row{background:#fcfcfc}tr.row-add td{padding:.2rem}tr.row-add td.action-column{padding:.5rem 1rem}tr td{box-sizing:border-box;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td.expand-column{padding:.3rem}tr td.column-hide{display:none}tr td.clear-left-border{border-left:0!important}tr td.clear-right-border{border-right:0!important}tr td select{border:1px solid #d1cece;height:2rem;padding:.2rem .5rem}"]
                },] }
    ];
    TreeBodyComponent.ctorParameters = function () { return [
        { type: AngularTreeGridService }
    ]; };
    TreeBodyComponent.propDecorators = {
        store: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        expand_tracker: [{ type: i0.Input }],
        edit_tracker: [{ type: i0.Input }],
        internal_configs: [{ type: i0.Input }],
        columns: [{ type: i0.Input }],
        cellclick: [{ type: i0.Input }],
        expand: [{ type: i0.Input }],
        collapse: [{ type: i0.Input }],
        rowdelete: [{ type: i0.Input }],
        rowsave: [{ type: i0.Input }],
        rowadd: [{ type: i0.Input }],
        rowselect: [{ type: i0.Input }],
        rowdeselect: [{ type: i0.Input }]
    };

    var TreeCellComponent = /** @class */ (function () {
        function TreeCellComponent() {
            this.rowexpand = new i0.EventEmitter();
            this.rowcollapse = new i0.EventEmitter();
            this.canceledit = new i0.EventEmitter();
            this.editcomplete = new i0.EventEmitter();
        }
        TreeCellComponent.prototype.ngOnInit = function () {
            this.is_expand_column = this.index === 0;
            this.show_expand_icon = !this.row_data.leaf;
            // If user mentions a node as leaf.
            if (this.configs.load_children_on_expand) {
                this.show_expand_icon = !this.row_data.leaf_node;
            }
            this.cell_value = this.row_data[this.column.name];
        };
        TreeCellComponent.prototype.expandRow = function (event) {
            if (this.index === 0 &&
                (!this.row_data.leaf || this.configs.load_children_on_expand)) {
                this.rowexpand.emit({ event: event, data: this.row_data });
                event.stopPropagation();
            }
        };
        TreeCellComponent.prototype.collapseRow = function (event) {
            if (this.index === 0 &&
                (!this.row_data.leaf || this.configs.load_children_on_expand)) {
                this.rowcollapse.emit({ event: event, data: this.row_data });
                event.stopPropagation();
            }
        };
        TreeCellComponent.prototype.onCellClick = function (event) {
            this.cellclick.emit({
                column: this.column,
                row: this.row_data,
                event: event,
            });
        };
        TreeCellComponent.prototype.onEditComplete = function ($event) {
            this.editcomplete.emit({ event: $event, data: this.row_data });
        };
        TreeCellComponent.prototype.getChildrenPadding = function () {
            var padding = this.row_data.leaf
                ? this.row_data.levelx * 10 + 20 + "px"
                : this.row_data.levelx * 10 + "px";
            return this.configs.rtl_direction
                ? {
                    "padding-right": padding,
                }
                : {
                    "padding-left": padding,
                };
        };
        return TreeCellComponent;
    }());
    TreeCellComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: "db-tree-cell",
                    template: "<div (click)=\"onCellClick($event)\">\n  <ng-container *ngIf=\"is_expand_column\">\n    <div\n      [ngStyle]=\"getChildrenPadding()\"\n    >\n      <span\n        class=\"no-expand-icon\"\n        *ngIf=\"row_data.levelx == 0 && row_data.leaf\"\n      ></span>\n      <ng-container *ngIf=\"!expand_tracker[row_data.pathx]\">\n        <span\n          (click)=\"expandRow($event)\"\n          class=\"expand-icon-container\"\n          *ngIf=\"show_expand_icon\"\n          title=\"Expand Row\"\n        >\n          <span\n            class=\"expand-icon\"\n            [ngStyle]=\"{\n              visibility: row_data.expand_disabled ? 'hidden' : 'visible'\n            }\"\n            *ngIf=\"show_expand_icon && configs.css.expand_icon.length > 0\"\n            [innerHTML]=\"configs.css.expand_icon | safeHtml\"\n          ></span>\n          <span\n            class=\"expand-icon inbuild-icon\"\n            [ngStyle]=\"{\n              visibility: row_data.expand_disabled ? 'hidden' : 'visible'\n            }\"\n            *ngIf=\"show_expand_icon && configs.css.expand_icon.length == 0\"\n            >+</span\n          >\n        </span>\n      </ng-container>\n      <ng-container *ngIf=\"expand_tracker[row_data.pathx]\">\n        <span *ngIf=\"row_data.is_loading\" class=\"childred-loader\"></span>\n        <span\n          (click)=\"collapseRow($event)\"\n          *ngIf=\"!row_data.is_loading && show_expand_icon\"\n          class=\"expand-icon-container\"\n          title=\"Collapse Row\"\n        >\n          <span\n            class=\"expand-icon\"\n            [ngStyle]=\"{\n              visibility: row_data.expand_disabled ? 'hidden' : 'visible'\n            }\"\n            *ngIf=\"show_expand_icon && configs.css.collapse_icon.length > 0\"\n            [innerHTML]=\"configs.css.collapse_icon | safeHtml\"\n          ></span>\n          <span\n            class=\"expand-icon inbuild-dash-icon\"\n            [ngStyle]=\"{\n              visibility: row_data.expand_disabled ? 'hidden' : 'visible'\n            }\"\n            *ngIf=\"show_expand_icon && configs.css.collapse_icon.length == 0\"\n            >\u2014</span\n          >\n        </span>\n      </ng-container>\n      <ng-container\n        *ngIf=\"!edit_on || !column.editable\"\n        [ngSwitch]=\"column.type\"\n      >\n        <db-custom-cell-component\n          *ngSwitchCase=\"'custom'\"\n          [column]=\"column\"\n          [row_data]=\"row_data\"\n        >\n        </db-custom-cell-component>\n        <db-tree-cell-view\n          *ngSwitchDefault\n          [column]=\"column\"\n          [row_data]=\"row_data\"\n        >\n        </db-tree-cell-view>\n      </ng-container>\n      <ng-container *ngIf=\"edit_on && column.editable\">\n        <db-custom-cell-editor-component\n          *ngIf=\"column.editor\"\n          [cell_value]=\"cell_value\"\n          [column]=\"column\"\n          [row_data]=\"row_data\"\n          (canceledit)=\"canceledit.emit(row_data['pathx'])\"\n          (editcomplete)=\"onEditComplete($event)\"\n        >\n        </db-custom-cell-editor-component>\n        <db-tree-cell-editor\n          *ngIf=\"!column.editor\"\n          [cell_value]=\"cell_value\"\n          [row_data]=\"row_data\"\n          [column]=\"column\"\n          [expandable_column]=\"true\"\n          (canceledit)=\"canceledit.emit(row_data['pathx'])\"\n          (editcomplete)=\"onEditComplete($event)\"\n        >\n        </db-tree-cell-editor>\n      </ng-container>\n    </div>\n  </ng-container>\n  <ng-container *ngIf=\"!is_expand_column\">\n    <ng-container *ngIf=\"!edit_on || !column.editable\" [ngSwitch]=\"column.type\">\n      <db-custom-cell-component\n        *ngSwitchCase=\"'custom'\"\n        [column]=\"column\"\n        [row_data]=\"row_data\"\n      >\n      </db-custom-cell-component>\n      <db-tree-cell-view\n        *ngSwitchDefault\n        [column]=\"column\"\n        [row_data]=\"row_data\"\n      >\n      </db-tree-cell-view>\n    </ng-container>\n\n    <ng-container *ngIf=\"edit_on && column.editable\">\n      <db-custom-cell-editor-component\n        *ngIf=\"column.editor\"\n        [cell_value]=\"cell_value\"\n        [column]=\"column\"\n        [row_data]=\"row_data\"\n        (canceledit)=\"canceledit.emit(row_data['pathx'])\"\n        (editcomplete)=\"onEditComplete($event)\"\n      >\n      </db-custom-cell-editor-component>\n      <db-tree-cell-editor\n        *ngIf=\"!column.editor\"\n        [cell_value]=\"cell_value\"\n        [row_data]=\"row_data\"\n        [column]=\"column\"\n        [expandable_column]=\"false\"\n        (canceledit)=\"canceledit.emit(row_data['pathx'])\"\n        (editcomplete)=\"onEditComplete($event)\"\n      >\n      </db-tree-cell-editor>\n    </ng-container>\n  </ng-container>\n</div>\n",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}i.expand-icon{margin-right:5px}span.no-expand-icon{display:inline-block;width:14px}span.expand-icon-container{box-sizing:border-box;cursor:pointer;padding:0 5px 0 0;position:relative;text-align:center;width:21px}span.expand-icon-container span.inbuild-icon{font-size:25px}span.expand-icon-container span.inbuild-dash-icon{font-size:17px;font-weight:700}span.childred-loader{animation:spin 1s linear infinite;border:3px solid #f3f3f3;border-radius:50%;border-top-color:#555;display:inline-block;height:10px;margin-right:5px;width:10px}@keyframes spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}"]
                },] }
    ];
    TreeCellComponent.ctorParameters = function () { return []; };
    TreeCellComponent.propDecorators = {
        configs: [{ type: i0.Input }],
        index: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        column: [{ type: i0.Input }],
        expand_tracker: [{ type: i0.Input }],
        cellclick: [{ type: i0.Input }],
        edit_on: [{ type: i0.Input }],
        rowexpand: [{ type: i0.Output }],
        rowcollapse: [{ type: i0.Output }],
        canceledit: [{ type: i0.Output }],
        editcomplete: [{ type: i0.Output }]
    };

    var TreeCellViewComponent = /** @class */ (function () {
        function TreeCellViewComponent() {
        }
        TreeCellViewComponent.prototype.ngOnInit = function () {
        };
        return TreeCellViewComponent;
    }());
    TreeCellViewComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'db-tree-cell-view',
                    template: "<span [innerHTML]=\"column.renderer ? column.renderer(row_data[column.name], row_data) : row_data[column.name]\"></span>\n",
                    styles: [""]
                },] }
    ];
    TreeCellViewComponent.ctorParameters = function () { return []; };
    TreeCellViewComponent.propDecorators = {
        column: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }]
    };

    var CustomCellViewComponent = /** @class */ (function () {
        function CustomCellViewComponent(resolver) {
            this.resolver = resolver;
        }
        CustomCellViewComponent.prototype.ngOnInit = function () {
            if (this.column.component && !this.custom_component) {
                this.createCustomComponent();
                this.callOnComponentInit();
            }
        };
        CustomCellViewComponent.prototype.ngOnDestroy = function () {
            if (this.custom_component) {
                this.custom_component.destroy();
            }
        };
        CustomCellViewComponent.prototype.createCustomComponent = function () {
            var componentFactory = this.resolver.resolveComponentFactory(this.column.component);
            this.custom_component = this.custom_view.createComponent(componentFactory);
        };
        CustomCellViewComponent.prototype.callOnComponentInit = function () {
            this.column.on_component_init && this.column.on_component_init(this.custom_component.instance);
            this.custom_component.instance.cell_value = this.row_data[this.column.name];
            this.custom_component.instance.row_data = this.row_data;
            this.custom_component.instance.column = this.column;
        };
        return CustomCellViewComponent;
    }());
    CustomCellViewComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'db-custom-cell-component',
                    template: "\n      <ng-template #customView></ng-template>\n    "
                },] }
    ];
    CustomCellViewComponent.ctorParameters = function () { return [
        { type: i0.ComponentFactoryResolver }
    ]; };
    CustomCellViewComponent.propDecorators = {
        column: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        custom_view: [{ type: i0.ViewChild, args: ['customView', { read: i0.ViewContainerRef, static: true },] }]
    };

    var TreeCellActionsComponent = /** @class */ (function () {
        function TreeCellActionsComponent() {
            this.editcomplete = new i0.EventEmitter();
            this.canceledit = new i0.EventEmitter();
        }
        TreeCellActionsComponent.prototype.ngOnInit = function () {
            this.display_data = this.store.getDisplayData();
        };
        TreeCellActionsComponent.prototype.enableEdit = function (index, row_data) {
            //Cancel all previous edits.
            for (var id in this.edit_tracker) {
                this.edit_tracker[id] = false;
            }
            this.edit_tracker[index] = true;
            // Only if edit_parent is true.
            if (this.configs.actions.edit_parent) {
                this.internal_configs.show_parent_col = true;
            }
            this.internal_configs.current_edited_row = Object.assign({}, row_data);
        };
        TreeCellActionsComponent.prototype.findRecordIndex = function (pathx) {
            for (var index in this.store.processed_data) {
                if (this.store.processed_data[index].pathx === pathx) {
                    return Number(index);
                }
            }
        };
        TreeCellActionsComponent.prototype.deleteRecord = function (rec) {
            var _this = this;
            var index = this.findRecordIndex(rec.pathx);
            if (this.configs.actions.resolve_delete) {
                var promise = new Promise(function (resolve, reject) {
                    _this.rowdelete.emit({
                        data: rec,
                        resolve: resolve
                    });
                });
                promise.then(function () {
                    _this.store.processed_data.splice(index, 1);
                    _this.store.refreshDisplayData();
                }).catch(function (err) { });
            }
            else {
                this.store.processed_data.splice(index, 1);
                this.store.refreshDisplayData();
                this.rowdelete.emit(rec);
            }
        };
        TreeCellActionsComponent.prototype.saveRecord = function ($event) {
            this.editcomplete.emit({ event: $event, data: this.row_data });
        };
        return TreeCellActionsComponent;
    }());
    TreeCellActionsComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-tree-cell-actions]',
                    template: "<ng-container\n  *ngIf=\"configs.actions.edit && configs.row_edit_function(row_data)\"\n>\n  <div *ngIf=\"!edit_tracker[row_data[configs.id_field]]\" class=\"icon-container\">\n    <span\n      (click)=\"enableEdit(row_data[configs.id_field], row_data)\"\n      *ngIf=\"configs.css.edit_icon\"\n      title=\"Edit row\"\n      [innerHTML]=\"this.configs.css.edit_icon | safeHtml\"\n    ></span>\n    <svg\n      (click)=\"enableEdit(row_data[configs.id_field], row_data)\"\n      *ngIf=\"!configs.css.edit_icon\"\n      xmlns=\"http://www.w3.org/2000/svg\"\n      viewBox=\"0 0 24 24\"\n    >\n      <title>Edit row</title>\n      <g data-name=\"Layer 2\">\n        <g data-name=\"edit\">\n          <rect width=\"24\" height=\"24\" opacity=\"0\" />\n          <path\n            d=\"M19.4 7.34L16.66 4.6A2 2 0 0 0 14 4.53l-9 9a2 2 0 0 0-.57 1.21L4 18.91a1 1 0 0 0 .29.8A1 1 0 0 0 5 20h.09l4.17-.38a2 2 0 0 0 1.21-.57l9-9a1.92 1.92 0 0 0-.07-2.71zM9.08 17.62l-3 .28.27-3L12 9.32l2.7 2.7zM16 10.68L13.32 8l1.95-2L18 8.73z\"\n          />\n        </g>\n      </g>\n    </svg>\n  </div>\n  <ng-container *ngIf=\"edit_tracker[row_data[configs.id_field]]\">\n    <div class=\"icon-container\">\n      <span\n        (click)=\"saveRecord($event)\"\n        *ngIf=\"configs.css.save_icon\"\n        title=\"Save row\"\n        [innerHTML]=\"this.configs.css.save_icon | safeHtml\"\n      ></span>\n      <svg\n        (click)=\"saveRecord($event)\"\n        title=\"Save row\"\n        *ngIf=\"!configs.css.save_icon\"\n        xmlns=\"http://www.w3.org/2000/svg\"\n        viewBox=\"0 0 24 24\"\n      >\n        <title>Save row</title>\n        <g data-name=\"Layer 2\">\n          <g data-name=\"save\">\n            <rect width=\"24\" height=\"24\" opacity=\"0\" />\n            <path\n              d=\"M20.12 8.71l-4.83-4.83A3 3 0 0 0 13.17 3H6a3 3 0 0 0-3 3v12a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3v-7.17a3 3 0 0 0-.88-2.12zM10 19v-2h4v2zm9-1a1 1 0 0 1-1 1h-2v-3a1 1 0 0 0-1-1H9a1 1 0 0 0-1 1v3H6a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h2v5a1 1 0 0 0 1 1h4a1 1 0 0 0 0-2h-3V5h3.17a1.05 1.05 0 0 1 .71.29l4.83 4.83a1 1 0 0 1 .29.71z\"\n            />\n          </g>\n        </g>\n      </svg>\n    </div>\n    <div class=\"icon-container\">\n      <span\n        (click)=\"canceledit.emit(row_data)\"\n        *ngIf=\"configs.css.cancel_icon\"\n        title=\"Cancel row\"\n        [innerHTML]=\"this.configs.css.cancel_icon | safeHtml\"\n      ></span>\n      <svg\n        (click)=\"canceledit.emit(row_data)\"\n        *ngIf=\"!configs.css.cancel_icon\"\n        title=\"Cancel row\"\n        xmlns=\"http://www.w3.org/2000/svg\"\n        viewBox=\"0 0 24 24\"\n      >\n        <title>Cancel row</title>\n        <g data-name=\"Layer 2\">\n          <g data-name=\"close\">\n            <rect\n              width=\"24\"\n              height=\"24\"\n              transform=\"rotate(180 12 12)\"\n              opacity=\"0\"\n            />\n            <path\n              d=\"M13.41 12l4.3-4.29a1 1 0 1 0-1.42-1.42L12 10.59l-4.29-4.3a1 1 0 0 0-1.42 1.42l4.3 4.29-4.3 4.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0l4.29-4.3 4.29 4.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42z\"\n            />\n          </g>\n        </g>\n      </svg>\n    </div>\n  </ng-container>\n</ng-container>\n<div\n  class=\"icon-container\"\n  *ngIf=\"\n    configs.actions.delete &&\n    !edit_tracker[row_data[configs.id_field]] &&\n    configs.row_delete_function(row_data)\n  \"\n>\n  <span\n    *ngIf=\"configs.css.delete_icon\"\n    title=\"Delete row\"\n    (click)=\"deleteRecord(row_data)\"\n    [innerHTML]=\"this.configs.css.delete_icon | safeHtml\"\n  ></span>\n  <svg\n    *ngIf=\"!configs.css.delete_icon\"\n    title=\"Delete row\"\n    (click)=\"deleteRecord(row_data)\"\n    xmlns=\"http://www.w3.org/2000/svg\"\n    viewBox=\"0 0 24 24\"\n  >\n    <title>Delete row</title>\n    <g data-name=\"Layer 2\">\n      <g data-name=\"trash-2\">\n        <rect width=\"24\" height=\"24\" opacity=\"0\" />\n        <path\n          d=\"M21 6h-5V4.33A2.42 2.42 0 0 0 13.5 2h-3A2.42 2.42 0 0 0 8 4.33V6H3a1 1 0 0 0 0 2h1v11a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V8h1a1 1 0 0 0 0-2zM10 4.33c0-.16.21-.33.5-.33h3c.29 0 .5.17.5.33V6h-4zM18 19a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V8h12z\"\n        />\n        <path d=\"M9 17a1 1 0 0 0 1-1v-4a1 1 0 0 0-2 0v4a1 1 0 0 0 1 1z\" />\n        <path d=\"M15 17a1 1 0 0 0 1-1v-4a1 1 0 0 0-2 0v4a1 1 0 0 0 1 1z\" />\n      </g>\n    </g>\n  </svg>\n</div>\n",
                    styles: ["svg{width:20px}div.icon-container{color:#000;cursor:pointer;display:inline-block;text-align:center;width:50%}"]
                },] }
    ];
    TreeCellActionsComponent.ctorParameters = function () { return []; };
    TreeCellActionsComponent.propDecorators = {
        store: [{ type: i0.Input }],
        edit_tracker: [{ type: i0.Input }],
        internal_configs: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        rowdelete: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        editcomplete: [{ type: i0.Output }],
        canceledit: [{ type: i0.Output }]
    };

    var DefaultEditor = /** @class */ (function () {
        function DefaultEditor() {
            this.canceledit = new i0.EventEmitter();
            this.editcomplete = new i0.EventEmitter();
            this.cellclick = new i0.EventEmitter();
        }
        return DefaultEditor;
    }());
    DefaultEditor.decorators = [
        { type: i0.Component, args: [{
                    selector: '',
                    template: ''
                },] }
    ];
    DefaultEditor.propDecorators = {
        canceledit: [{ type: i0.Output }],
        editcomplete: [{ type: i0.Output }],
        cellclick: [{ type: i0.Output }]
    };

    var CustomCellEditorComponent = /** @class */ (function (_super) {
        __extends(CustomCellEditorComponent, _super);
        function CustomCellEditorComponent(resolver) {
            var _this = _super.call(this) || this;
            _this.resolver = resolver;
            return _this;
        }
        CustomCellEditorComponent.prototype.ngOnInit = function () {
            if (this.column.editor && !this.custom_component) {
                this.createCustomComponent();
                this.callOnComponentInit();
            }
        };
        CustomCellEditorComponent.prototype.ngOnDestroy = function () {
            if (this.custom_component) {
                this.custom_component.destroy();
            }
        };
        CustomCellEditorComponent.prototype.createCustomComponent = function () {
            var componentFactory = this.resolver.resolveComponentFactory(this.column.editor);
            this.custom_component = this.custom_view.createComponent(componentFactory);
        };
        CustomCellEditorComponent.prototype.callOnComponentInit = function () {
            var _this = this;
            this.column.on_component_init && this.column.on_component_init(this.custom_component.instance);
            this.custom_component.instance.cell_value = this.cell_value;
            this.custom_component.instance.row_data = this.row_data;
            this.custom_component.instance.column = this.column;
            this.custom_component.instance.editcomplete.subscribe(function (event) { return _this.editcomplete.emit(event); });
            this.custom_component.instance.canceledit.subscribe(function (event) { return _this.canceledit.emit(event); });
            this.custom_component.instance.cellclick.subscribe(function (event) { return _this.cellclick.emit(event); });
        };
        return CustomCellEditorComponent;
    }(DefaultEditor));
    CustomCellEditorComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'db-custom-cell-editor-component',
                    template: "\n      <ng-template #customView></ng-template>\n    "
                },] }
    ];
    CustomCellEditorComponent.ctorParameters = function () { return [
        { type: i0.ComponentFactoryResolver }
    ]; };
    CustomCellEditorComponent.propDecorators = {
        column: [{ type: i0.Input }],
        cell_value: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        custom_view: [{ type: i0.ViewChild, args: ['customView', { read: i0.ViewContainerRef, static: true },] }]
    };

    var TreeCellEditorComponent = /** @class */ (function (_super) {
        __extends(TreeCellEditorComponent, _super);
        function TreeCellEditorComponent() {
            return _super.call(this) || this;
        }
        TreeCellEditorComponent.prototype.ngOnInit = function () {
        };
        return TreeCellEditorComponent;
    }(DefaultEditor));
    TreeCellEditorComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'db-tree-cell-editor',
                    template: "<input type=\"text\" \n    [(ngModel)]=\"row_data[column.name]\" \n    [ngClass]=\"{'cell-default': true, 'expandable-editor': expandable_column}\"\n    (click)=\"cellclick.emit($event)\"\n    (keydown.enter)=\"editcomplete.emit($event)\"\n    (keydown.esc)=\"canceledit.emit()\"\n    size=\"1\"\n    >",
                    styles: ["input.cell-default{border:1px solid #d1cece;border-radius:3px;box-sizing:border-box;height:2rem;padding:.2rem .5rem;width:100%}input.expandable-editor{width:calc(100% - 18px)}"]
                },] }
    ];
    TreeCellEditorComponent.ctorParameters = function () { return []; };
    TreeCellEditorComponent.propDecorators = {
        cell_value: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        column: [{ type: i0.Input }],
        expandable_column: [{ type: i0.Input }]
    };

    var SafeHtmlPipe = /** @class */ (function () {
        function SafeHtmlPipe(sanitized) {
            this.sanitized = sanitized;
        }
        SafeHtmlPipe.prototype.transform = function (value) {
            return this.sanitized.bypassSecurityTrustHtml(value);
        };
        return SafeHtmlPipe;
    }());
    SafeHtmlPipe.decorators = [
        { type: i0.Pipe, args: [{ name: 'safeHtml' },] }
    ];
    SafeHtmlPipe.ctorParameters = function () { return [
        { type: platformBrowser.DomSanitizer }
    ]; };

    var TreeCellModule = /** @class */ (function () {
        function TreeCellModule() {
        }
        return TreeCellModule;
    }());
    TreeCellModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [
                        TreeCellComponent,
                        TreeCellViewComponent,
                        CustomCellViewComponent,
                        TreeCellActionsComponent,
                        CustomCellEditorComponent,
                        TreeCellEditorComponent,
                        SafeHtmlPipe
                    ],
                    imports: [
                        common.CommonModule,
                        forms.FormsModule
                    ],
                    exports: [
                        TreeCellComponent,
                        CustomCellViewComponent,
                        CustomCellEditorComponent,
                        TreeCellEditorComponent,
                        TreeCellActionsComponent,
                        SafeHtmlPipe
                    ]
                },] }
    ];

    var AddRowComponent = /** @class */ (function () {
        function AddRowComponent() {
            this.row_data = {};
            this.parents = [];
            this.rowadd = new i0.EventEmitter();
            this.canceledit = new i0.EventEmitter();
        }
        AddRowComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.raw_data = this.store.getRawData();
            this.columns.forEach(function (column) {
                _this.row_data[column.name] = '';
            });
            this.parents = this.raw_data.map(function (element) {
                return {
                    'id': element[_this.configs.id_field],
                    'value': element[_this.configs.parent_display_field]
                };
            });
        };
        AddRowComponent.prototype.saveAddRecord = function (e) {
            this.raw_data.push(this.row_data);
            this.rowadd.emit(this.row_data);
        };
        AddRowComponent.prototype.cancelAddEdit = function () {
            this.internal_configs.show_add_row = false;
        };
        return AddRowComponent;
    }());
    AddRowComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-add-row]',
                    template: "<td *ngIf=\"configs.multi_select\"></td>\n<td class=\"action-column\">\n  <div class=\"icon-container\">\n    <span\n      (click)=\"saveAddRecord($event)\"\n      *ngIf=\"configs.css.save_icon\"\n      title=\"Save row\"\n      [innerHTML]=\"this.configs.css.save_icon | safeHtml\"\n    ></span>\n    <svg\n      (click)=\"saveAddRecord($event)\"\n      title=\"Save row\"\n      *ngIf=\"!configs.css.save_icon\"\n      xmlns=\"http://www.w3.org/2000/svg\"\n      viewBox=\"0 0 24 24\"\n    >\n      <title>Save row</title>\n      <g data-name=\"Layer 2\">\n        <g data-name=\"save\">\n          <rect width=\"24\" height=\"24\" opacity=\"0\" />\n          <path\n            d=\"M20.12 8.71l-4.83-4.83A3 3 0 0 0 13.17 3H6a3 3 0 0 0-3 3v12a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3v-7.17a3 3 0 0 0-.88-2.12zM10 19v-2h4v2zm9-1a1 1 0 0 1-1 1h-2v-3a1 1 0 0 0-1-1H9a1 1 0 0 0-1 1v3H6a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h2v5a1 1 0 0 0 1 1h4a1 1 0 0 0 0-2h-3V5h3.17a1.05 1.05 0 0 1 .71.29l4.83 4.83a1 1 0 0 1 .29.71z\"\n          />\n        </g>\n      </g>\n    </svg>\n  </div>\n  <div class=\"icon-container\">\n    <span\n      (click)=\"cancelAddEdit()\"\n      *ngIf=\"configs.css.cancel_icon\"\n      title=\"Cancel row\"\n      [innerHTML]=\"this.configs.css.cancel_icon | safeHtml\"\n    ></span>\n    <svg\n      (click)=\"cancelAddEdit()\"\n      *ngIf=\"!configs.css.cancel_icon\"\n      title=\"Cancel row\"\n      xmlns=\"http://www.w3.org/2000/svg\"\n      viewBox=\"0 0 24 24\"\n    >\n      <title>Cancel row</title>\n      <g data-name=\"Layer 2\">\n        <g data-name=\"close\">\n          <rect\n            width=\"24\"\n            height=\"24\"\n            transform=\"rotate(180 12 12)\"\n            opacity=\"0\"\n          />\n          <path\n            d=\"M13.41 12l4.3-4.29a1 1 0 1 0-1.42-1.42L12 10.59l-4.29-4.3a1 1 0 0 0-1.42 1.42l4.3 4.29-4.3 4.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0l4.29-4.3 4.29 4.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42z\"\n          />\n        </g>\n      </g>\n    </svg>\n  </div>\n</td>\n<td *ngFor=\"let column of columns\" [ngClass]=\"{ 'column-hide': column.hidden }\">\n  <div *ngIf=\"column.editable\">\n    <db-custom-cell-editor-component\n      *ngIf=\"column.editor\"\n      [cell_value]=\"\"\n      [column]=\"column\"\n      [row_data]=\"row_data\"\n      (canceledit)=\"cancelAddEdit()\"\n      (editcomplete)=\"saveAddRecord($event)\"\n    >\n    </db-custom-cell-editor-component>\n    <db-tree-cell-editor\n      *ngIf=\"!column.editor\"\n      [cell_value]=\"\"\n      [row_data]=\"row_data\"\n      [column]=\"column\"\n      (canceledit)=\"cancelAddEdit()\"\n      (editcomplete)=\"saveAddRecord($event)\"\n    >\n    </db-tree-cell-editor>\n  </div>\n</td>\n<td>\n  <select [(ngModel)]=\"row_data[configs.parent_id_field]\">\n    <option *ngFor=\"let parent of parents\" [value]=\"parent.id\">\n      {{ parent.value }}\n    </option>\n  </select>\n</td>\n",
                    encapsulation: i0.ViewEncapsulation.None,
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{border-bottom:1px solid #cdd5dc}tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr.subgrid-row{background:#fcfcfc}tr.row-add td{padding:.2rem}tr.row-add td.action-column{padding:.5rem 1rem}tr td{box-sizing:border-box;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td.expand-column{padding:.3rem}tr td.column-hide{display:none}tr td.clear-left-border{border-left:0!important}tr td.clear-right-border{border-right:0!important}tr td select{border:1px solid #d1cece;height:2rem;padding:.2rem .5rem}svg{padding-right:4px;width:20px}div.icon-container{display:inline-block;text-align:center;width:50%}"]
                },] }
    ];
    AddRowComponent.ctorParameters = function () { return []; };
    AddRowComponent.propDecorators = {
        store: [{ type: i0.Input }],
        columns: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        internal_configs: [{ type: i0.Input }],
        rowadd: [{ type: i0.Output }],
        canceledit: [{ type: i0.Output }]
    };

    var FilterRowComponent = /** @class */ (function () {
        function FilterRowComponent(angularTreeGridService) {
            this.angularTreeGridService = angularTreeGridService;
            this.search_values = {};
        }
        FilterRowComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.columns.forEach(function (column) {
                _this.search_values[column.name] = '';
            });
        };
        FilterRowComponent.prototype.filter = function () {
            this.store.filterBy(this.columns, Object.values(this.search_values));
            // Don't expand for subgrid.
            if (!this.configs.subgrid) {
                this.angularTreeGridService.expandAll(this.expand_tracker);
            }
        };
        return FilterRowComponent;
    }());
    FilterRowComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-filter-row]',
                    template: "<td *ngIf=\"configs.multi_select\"></td>\n<td *ngIf=\"(configs.actions.edit || configs.actions.delete || configs.actions.add)\"></td>\n<td *ngFor=\"let column of columns\">\n  <div *ngIf=\"column.filter\">\n    <input type=\"text\" (keyup)=\"filter()\" [(ngModel)]=\"search_values[column.name]\">\n  </div>\n</td>\n<!-- Add column to fix UI issue when add row is enabled or Edit is enabled -->\n<td *ngIf=\"internal_configs.show_add_row || (internal_configs.show_parent_col && configs.show_parent_on_edit)\"></td>",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{border-bottom:1px solid #cdd5dc}tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr.subgrid-row{background:#fcfcfc}tr.row-add td{padding:.2rem}tr.row-add td.action-column{padding:.5rem 1rem}tr td{box-sizing:border-box;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td.expand-column{padding:.3rem}tr td.column-hide{display:none}tr td.clear-left-border{border-left:0!important}tr td.clear-right-border{border-right:0!important}tr td select{border:1px solid #d1cece;height:2rem;padding:.2rem .5rem}td{padding:.2rem;position:relative;vertical-align:middle}td input{-moz-box-sizing:border-box;-webkit-box-sizing:border-box;border:1px solid #cdd5dc;border-radius:3px;box-sizing:border-box;height:2rem;padding:.2rem .5rem;width:100%}"]
                },] }
    ];
    FilterRowComponent.ctorParameters = function () { return [
        { type: AngularTreeGridService }
    ]; };
    FilterRowComponent.propDecorators = {
        store: [{ type: i0.Input }],
        columns: [{ type: i0.Input }],
        expand_tracker: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        internal_configs: [{ type: i0.Input }]
    };

    var SubgridComponent = /** @class */ (function () {
        function SubgridComponent() {
        }
        SubgridComponent.prototype.ngOnInit = function () { };
        SubgridComponent.prototype.saveRecord = function ($event) {
            var _this = this;
            var element = $event.data;
            if (this.configs.actions.resolve_edit) {
                var promise = new Promise(function (resolve, reject) {
                    _this.rowsave.emit({
                        data: element,
                        resolve: resolve
                    });
                });
                promise.then(function () {
                    _this.checkAndRefreshData(element);
                }).catch(function (err) { });
            }
            else {
                this.checkAndRefreshData(element);
                this.rowsave.emit(element);
            }
        };
        SubgridComponent.prototype.checkAndRefreshData = function (element) {
            this.edit_tracker[element[this.configs.id_field]] = false;
            this.internal_configs.show_parent_col = false;
            // Only refresh if Parent has been changed.
            if (this.internal_configs.current_edited_row[this.configs.parent_id_field]
                !== element[this.configs.parent_id_field]) {
                this.refreshData(element);
            }
        };
        SubgridComponent.prototype.refreshData = function (element) {
            // If edit parent is not true then don't refresh.
            if (!this.configs.actions.edit_parent) {
                return;
            }
            element[this.configs.parent_id_field] = parseInt(element[this.configs.parent_id_field], 10);
            this.expand_tracker = {};
            this.edit_tracker = {};
            this.store.processData(this.store.getRawData(), this.expand_tracker, this.configs, this.edit_tracker, this.internal_configs);
        };
        SubgridComponent.prototype.cancelEdit = function (row_data) {
            var index = row_data[this.configs.id_field];
            // Cancel all changes ie copy from back up.
            Object.assign(row_data, this.internal_configs.current_edited_row);
            this.edit_tracker[index] = false;
            this.internal_configs.show_parent_col = false;
        };
        SubgridComponent.prototype.onRowExpand = function (event) {
            var _this = this;
            var row_data = event.data;
            var promise = new Promise(function (resolve, reject) {
                _this.expand.emit({
                    data: row_data,
                    resolve: resolve
                });
            });
            this.expand_tracker[row_data.pathx] = true;
            var blank_row = this.store.showBlankRow(row_data);
            blank_row.loading_children = true;
            // Add Child rows to the table.
            promise.then(function (child_rows) {
                blank_row.loading_children = false;
                if (child_rows) {
                    child_rows.map(function (child) {
                        child.leaf = true;
                    });
                    blank_row.children = child_rows;
                }
                else {
                    // Persist old children. If didn't exist then assign blank array.
                    if (!blank_row.children) {
                        blank_row.children = [];
                    }
                }
            }).catch(function (err) { });
        };
        SubgridComponent.prototype.onRowCollapse = function (event) {
            var row_data = event.data;
            this.expand_tracker[row_data.pathx] = false;
        };
        SubgridComponent.prototype.selectRowOnCheck = function (row_data, event) {
            if (event.target.checked) {
                row_data.row_selected = true;
                this.rowselect.emit({ data: row_data, event: event });
            }
            else {
                row_data.row_selected = false;
                this.rowdeselect.emit({ data: row_data, event: event });
            }
            this.setSelectAllConfig();
        };
        /**
         * Set Select All config on Select change.
         *
         */
        SubgridComponent.prototype.setSelectAllConfig = function () {
            var select_all = true;
            this.store.getDisplayData().forEach(function (data) {
                if (!data.row_selected) {
                    select_all = false;
                }
            });
            this.internal_configs.all_selected = select_all;
        };
        return SubgridComponent;
    }());
    SubgridComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-subgrid]',
                    template: "<ng-container *ngIf=\"expand_tracker[row_data.parent_pathx]\">\n\n  <!-- Expandable Row -->\n  <ng-container *ngIf=\"!row_data.leaf\">\n    <td *ngIf=\"configs.multi_select\" class=\"checkbox_column\">\n      <input type=\"checkbox\" [checked]=\"row_data.row_selected\" (click)=\"selectRowOnCheck(row_data, $event)\" \n        [attr.disabled]=\"row_data.selection_disabled\">\n    </td>\n    <td db-tree-cell-actions \n      *ngIf=\"(configs.actions.edit || configs.actions.delete || configs.actions.add)\"\n      [row_data]=\"row_data\"\n      [configs]=\"configs\"\n      [store]=\"store\"\n      [edit_tracker]=\"edit_tracker\"\n      [internal_configs]=\"internal_configs\"\n      [rowdelete]=\"rowdelete\"\n      (canceledit)=\"cancelEdit($event)\" \n      (editcomplete)=\"saveRecord($event)\"\n      >\n    </td>\n    <td *ngFor=\"let column of configs.columns; index as i\" \n      [ngClass]=\"{'column-hide': column.hidden, 'expand-column': i == 0}\">\n      <db-tree-cell\n        [configs]=\"configs\"\n        [column]=\"column\"\n        [index]=\"i\"\n        [row_data]=\"row_data\"\n        [expand_tracker]=\"expand_tracker\"\n        [edit_on]=\"edit_tracker[row_data[configs.id_field]]\"\n        [cellclick]=\"cellclick\"\n        (rowexpand)=\"onRowExpand($event)\"\n        (rowcollapse)=\"onRowCollapse($event)\"\n        (editcomplete)=\"saveRecord($event)\"\n      >\n      </db-tree-cell>\n    </td>\n  </ng-container>\n\n  <!-- Subgrid Rows -->\n  <ng-container *ngIf=\"row_data.leaf\">    \n    <td *ngIf=\"configs.multi_select\"></td>\n    <td *ngIf=\"(configs.actions.edit || configs.actions.delete || configs.actions.add)\"></td>\n    <td [attr.colspan]=\"configs.columns.length\" class=\"subgrid-column\">\n        <table class=\"subgrid-table\">\n          <thead db-subgrid-head\n            [row_data]=\"row_data\"\n            [configs]=\"configs\">\n            \n          </thead>\n          <tbody db-subgrid-body\n            [configs]=\"configs\"\n            [expand_tracker]=\"expand_tracker\"\n            [cellclick]=\"cellclick\"\n            [row_data]=\"row_data\">\n          </tbody>            \n        </table>\n    </td>    \n  </ng-container>  \n</ng-container>",
                    styles: ["tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr.subgrid-row{background:#fcfcfc}tr.row-add td{padding:.2rem}tr.row-add td.action-column{padding:.5rem 1rem}tr td{box-sizing:border-box;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td.expand-column{padding:.3rem}tr td.column-hide{display:none}tr td.clear-left-border{border-left:0!important}tr td.clear-right-border{border-right:0!important}tr td select{border:1px solid #d1cece;height:2rem;padding:.2rem .5rem}.db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{background:#fff;border-bottom:1px solid #cdd5dc}tr th{box-sizing:border-box;color:#181818;font-size:1rem;font-weight:600;line-height:1.25;position:relative;vertical-align:middle}tr th div{padding:.5rem}tr th.column-hide{display:none}tr th.action-column span.icon-container{cursor:pointer}tr th span.inbuild-icon{font-size:25px}th.clear-left-border{border-left:0!important}th.clear-right-border{border-right:0!important}.column-hide{display:none}svg{padding-right:3px;width:25px}td{border:1px solid #cdd5dc;box-sizing:border-box;padding:.5rem;position:relative;vertical-align:middle}td.checkbox_column{text-align:center}td.expand-column{padding:.3rem}td.subgrid-column{padding:0}td.column-hide{display:none}td.clear-left-border{border-left:0!important}td.clear-right-border{border-right:0!important}td table.subgrid-table{border-collapse:collapse;width:100%}"]
                },] }
    ];
    SubgridComponent.ctorParameters = function () { return []; };
    SubgridComponent.propDecorators = {
        store: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        expand_tracker: [{ type: i0.Input }],
        edit_tracker: [{ type: i0.Input }],
        internal_configs: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        cellclick: [{ type: i0.Input }],
        expand: [{ type: i0.Input }],
        rowselect: [{ type: i0.Input }],
        rowdeselect: [{ type: i0.Input }],
        rowsave: [{ type: i0.Input }],
        rowdelete: [{ type: i0.Input }]
    };

    var SubgridHeadComponent = /** @class */ (function () {
        function SubgridHeadComponent() {
        }
        SubgridHeadComponent.prototype.ngOnInit = function () {
        };
        SubgridHeadComponent.prototype.sortColumn = function (row_data, column) {
            var sort_by = column.name;
            // If already sorted then reverse.
            column.sort_type = column.sorted ? !column.sort_type : 1;
            column.sorted = 1;
            column.sort_type ? row_data.children.sort(function (a, b) { return (a[sort_by] > b[sort_by]) ? 1 : ((b[sort_by] > a[sort_by]) ? -1 : 0); }) :
                row_data.children.sort(function (a, b) { return (a[sort_by] < b[sort_by]) ? 1 : ((b[sort_by] < a[sort_by]) ? -1 : 0); });
        };
        return SubgridHeadComponent;
    }());
    SubgridHeadComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-subgrid-head]',
                    template: "<tr>\n  <th *ngFor=\"let column of configs.subgrid_config.columns\"\n    (click)=\"sortColumn(row_data, column)\" width=\"{{column.width ? column.width : 'auto'}}\"\n    [ngClass]=\"{'sorted': column.sorted,'sortable': column.sortable, 'column-hide': column.hidden}\">\n    {{column.header}}\n    <span *ngIf=\"column.sortable\" [ngClass]=\"column.sorted && column.sort_type == 0?'arrow-down active':'arrow-down'\"></span>\n    <span *ngIf=\"column.sortable\" [ngClass]=\"column.sorted && column.sort_type == 1?'arrow-up active':'arrow-up'\"></span>\n  </th>\n</tr>",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{background:#fff;border-bottom:1px solid #cdd5dc}tr th{box-sizing:border-box;color:#181818;font-size:1rem;font-weight:600;line-height:1.25;position:relative;vertical-align:middle}tr th div{padding:.5rem}tr th.column-hide{display:none}tr th.action-column span.icon-container{cursor:pointer}tr th span.inbuild-icon{font-size:25px}th.clear-left-border{border-left:0!important}th.clear-right-border{border-right:0!important}.column-hide{display:none}svg{padding-right:3px;width:25px}th.sortable{cursor:pointer}th.sortable:hover span.arrow-up{border-bottom:6px solid #afafaf}th.sortable:hover span.arrow-down{border-top:6px solid #afafaf}th span.arrow-up{border-bottom:6px solid #ddd;border-left:5px solid transparent;border-right:5px solid transparent;height:0;position:relative;top:-18px;width:0}th span.arrow-up.active{border-bottom:6px solid #8a8989!important}th span.arrow-down{border-left:5px solid transparent;border-right:5px solid transparent;border-top:6px solid #ddd;height:0;left:10px;position:relative;top:17px;width:0}th span.arrow-down.active{border-top:6px solid #8a8989!important}"]
                },] }
    ];
    SubgridHeadComponent.ctorParameters = function () { return []; };
    SubgridHeadComponent.propDecorators = {
        configs: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }]
    };

    var SubgridBodyComponent = /** @class */ (function () {
        function SubgridBodyComponent() {
        }
        SubgridBodyComponent.prototype.ngOnInit = function () {
        };
        return SubgridBodyComponent;
    }());
    SubgridBodyComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-subgrid-body]',
                    template: "<ng-container *ngIf=\"row_data.loading_children\">\n  <tr>\n    <td [attr.colspan]=\"configs.subgrid_config.columns.length\" \n      [innerHTML]=\"configs.subgrid_config.data_loading_text\"\n      class=\"subgrid-loading-text\"></td>\n  </tr>\n</ng-container>\n<ng-container *ngIf=\"!row_data.loading_children\">\n  <tr *ngFor=\"let child_data of row_data.children\">\n    <td *ngFor=\"let column of configs.subgrid_config.columns; index as i\">\n      <db-tree-cell\n        [configs]=\"configs\"\n        [column]=\"column\"\n        [index]=\"1\"\n        [row_data]=\"child_data\"\n        [expand_tracker]=\"expand_tracker\"\n        [cellclick]=\"cellclick\"\n      ></db-tree-cell>\n    </td>\n  </tr>\n  <tr *ngIf=\"configs.subgrid_config.show_summary_row\">\n    <td *ngFor=\"let column of configs.subgrid_config.columns\">\n      <div [innerHTML]=\"column.summary_renderer && column.summary_renderer(row_data.children)\"></div>\n    </td>\n  </tr>\n</ng-container>",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{border-bottom:1px solid #cdd5dc}tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr.subgrid-row{background:#fcfcfc}tr.row-add td{padding:.2rem}tr.row-add td.action-column{padding:.5rem 1rem}tr td{box-sizing:border-box;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td.expand-column{padding:.3rem}tr td.column-hide{display:none}tr td.clear-left-border{border-left:0!important}tr td.clear-right-border{border-right:0!important}tr td select{border:1px solid #d1cece;height:2rem;padding:.2rem .5rem}td.subgrid-loading-text{text-align:center}"]
                },] }
    ];
    SubgridBodyComponent.ctorParameters = function () { return []; };
    SubgridBodyComponent.propDecorators = {
        configs: [{ type: i0.Input }],
        expand_tracker: [{ type: i0.Input }],
        row_data: [{ type: i0.Input }],
        cellclick: [{ type: i0.Input }]
    };

    var SubgridModule = /** @class */ (function () {
        function SubgridModule() {
        }
        return SubgridModule;
    }());
    SubgridModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [SubgridComponent, SubgridHeadComponent, SubgridBodyComponent],
                    imports: [
                        common.CommonModule,
                        TreeCellModule
                    ],
                    exports: [SubgridComponent]
                },] }
    ];

    var TreeBodyModule = /** @class */ (function () {
        function TreeBodyModule() {
        }
        return TreeBodyModule;
    }());
    TreeBodyModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [TreeBodyComponent, AddRowComponent, FilterRowComponent],
                    imports: [
                        common.CommonModule,
                        TreeCellModule,
                        forms.FormsModule,
                        SubgridModule
                    ],
                    exports: [TreeBodyComponent]
                },] }
    ];

    var TreeHeadComponent = /** @class */ (function () {
        function TreeHeadComponent(angularTreeGridService) {
            this.angularTreeGridService = angularTreeGridService;
        }
        TreeHeadComponent.prototype.ngOnInit = function () { };
        TreeHeadComponent.prototype.addRow = function () {
            this.internal_configs.show_add_row = true;
        };
        TreeHeadComponent.prototype.selectAll = function (e) {
            if (e.target.checked) {
                this.angularTreeGridService.selectAll(this.store.getDisplayData());
                this.rowselectall.emit(this.store.getDisplayData());
            }
            else {
                this.angularTreeGridService.deSelectAll(this.store.getDisplayData());
                this.rowdeselectall.emit(e);
            }
        };
        return TreeHeadComponent;
    }());
    TreeHeadComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: '[db-tree-head]',
                    template: "<ng-container *ngIf=\"configs\">\n  <tr [ngClass]=\"configs.css.header_class\">\n    <th *ngIf=\"configs.multi_select\" class=\"checkbox_column\">\n      <input\n        type=\"checkbox\"\n        (click)=\"selectAll($event)\"\n        [checked]=\"this.internal_configs.all_selected\"\n      />\n    </th>\n    <th\n      *ngIf=\"\n        configs.actions.add || configs.actions.edit || configs.actions.delete\n      \"\n      class=\"action-column\"\n      width=\"{{ configs.action_column_width }}\"\n      (click)=\"addRow()\"\n    >\n      <span class=\"icon-container\">\n        <span\n          *ngIf=\"\n            !internal_configs.show_add_row &&\n            configs.actions.add &&\n            configs.css.add_icon.length > 0\n          \"\n          [innerHTML]=\"configs.css.add_icon | safeHtml\"\n          title=\"Add a row\"\n        ></span>\n        <span\n          *ngIf=\"\n            !internal_configs.show_add_row &&\n            configs.actions.add &&\n            configs.css.add_icon.length == 0\n          \"\n          title=\"Add a row\"\n          class=\"inbuild-icon\"\n          >+</span\n        >\n      </span>\n      <span *ngIf=\"internal_configs.show_add_row || !configs.actions.add\"\n        >Actions</span\n      >\n    </th>\n    <th\n      *ngFor=\"let column of columns\"\n      [ngClass]=\"{ 'column-hide': column.hidden }\"\n      [attr.width]=\"column.width\"\n    >\n      <div\n        [innerHTML]=\"\n          column.header_renderer\n            ? column.header_renderer(column.header)\n            : column.header\n        \"\n      ></div>\n    </th>\n    <th\n      *ngIf=\"\n        (configs.show_parent_on_edit && internal_configs.show_parent_col) ||\n        internal_configs.show_add_row\n      \"\n    >\n      Parent\n    </th>\n  </tr>\n</ng-container>\n",
                    styles: [".db-tree-view{border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{background:#fff;border-bottom:1px solid #cdd5dc}tr th{box-sizing:border-box;color:#181818;font-size:1rem;font-weight:600;line-height:1.25;position:relative;vertical-align:middle}tr th div{padding:.5rem}tr th.column-hide{display:none}tr th.action-column span.icon-container{cursor:pointer}tr th span.inbuild-icon{font-size:25px}th.clear-left-border{border-left:0!important}th.clear-right-border{border-right:0!important}.column-hide{display:none}svg{padding-right:3px;width:25px}"]
                },] }
    ];
    TreeHeadComponent.ctorParameters = function () { return [
        { type: AngularTreeGridService }
    ]; };
    TreeHeadComponent.propDecorators = {
        store: [{ type: i0.Input }],
        configs: [{ type: i0.Input }],
        expand_tracker: [{ type: i0.Input }],
        edit_tracker: [{ type: i0.Input }],
        internal_configs: [{ type: i0.Input }],
        columns: [{ type: i0.Input }],
        rowselectall: [{ type: i0.Input }],
        rowdeselectall: [{ type: i0.Input }]
    };

    var TreeHeadModule = /** @class */ (function () {
        function TreeHeadModule() {
        }
        return TreeHeadModule;
    }());
    TreeHeadModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [TreeHeadComponent],
                    imports: [
                        common.CommonModule
                    ],
                    exports: [TreeHeadComponent]
                },] }
    ];

    var AngularTreeGridModule = /** @class */ (function () {
        function AngularTreeGridModule() {
        }
        return AngularTreeGridModule;
    }());
    AngularTreeGridModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [AngularTreeGridComponent],
                    imports: [
                        common.CommonModule,
                        TreeBodyModule,
                        TreeHeadModule
                    ],
                    exports: [AngularTreeGridComponent]
                },] }
    ];

    /*
     * Public API Surface of angular-tree-grid
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.AngularTreeGridComponent = AngularTreeGridComponent;
    exports.AngularTreeGridModule = AngularTreeGridModule;
    exports.AngularTreeGridService = AngularTreeGridService;
    exports.DefaultEditor = DefaultEditor;
    exports.ɵa = TreeBodyModule;
    exports.ɵb = TreeBodyComponent;
    exports.ɵc = AddRowComponent;
    exports.ɵd = FilterRowComponent;
    exports.ɵe = TreeCellModule;
    exports.ɵf = TreeCellComponent;
    exports.ɵg = TreeCellViewComponent;
    exports.ɵh = CustomCellViewComponent;
    exports.ɵi = TreeCellActionsComponent;
    exports.ɵj = CustomCellEditorComponent;
    exports.ɵk = TreeCellEditorComponent;
    exports.ɵl = SafeHtmlPipe;
    exports.ɵm = SubgridModule;
    exports.ɵn = SubgridComponent;
    exports.ɵo = SubgridHeadComponent;
    exports.ɵp = SubgridBodyComponent;
    exports.ɵq = TreeHeadModule;
    exports.ɵr = TreeHeadComponent;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=angular-tree-grid.umd.js.map
